@@ -12,10 +12,12 @@
 #include <cmath>     // for isnan
 #include <algorithm> // for sort, any_of
 #include <array>
+#include <initializer_list>
 #include <string>
 #include <vector>
-#include "atox.hpp"      // for simple_atof, simple_atoi, read_word
-#include "input.hpp"     // for FileStream
+#include "atox.hpp"      // for simple_atoi, read_word
+#include "atof.hpp"      // for fast_atof
+#include "input.hpp"     // for FileStream, CharArray
 #include "iterator.hpp"  // for StrideIter
 #include "fail.hpp"      // for fail
 #include "fileutil.hpp"  // for file_open, is_little_endian, fileptr_t, ...
@@ -142,6 +144,7 @@ struct Mtz {
     }
 
     int dataset_id() const { return ints[20]; }
+    void set_dataset_id(int id) { ints[20] = id; }
     float phi_start() const { return floats[36]; }
     float phi_end() const { return floats[37]; }
     Mat33 matrix_U() const {
@@ -151,6 +154,7 @@ struct Mtz {
     }
   };
 
+  std::string source_path;  // input file path, if known
   bool same_byte_order = true;
   std::int32_t header_offset = 0;
   std::string version_stamp;
@@ -170,6 +174,7 @@ struct Mtz {
   std::vector<Column> columns;
   std::vector<Batch> batches;
   std::vector<std::string> history;
+  std::string appended_text;
   std::vector<float> data;
 
   FILE* warnings = nullptr;
@@ -202,6 +207,7 @@ struct Mtz {
     columns = std::move(o.columns);
     batches = std::move(o.batches);
     history = std::move(o.history);
+    appended_text = std::move(o.appended_text);
     data = std::move(o.data);
     warnings = o.warnings;
     for (Mtz::Column& col : columns)
@@ -234,6 +240,26 @@ struct Mtz {
       ds.cell = cell;
   }
 
+  UnitCell get_average_cell_from_batch_headers(double* rmsd) const {
+    double avg[6] = {0., 0., 0., 0., 0., 0.};
+    for (const Batch& batch : batches)
+      for (int i = 0; i < 6; ++i)
+        avg[i] += batch.floats[i];
+    size_t n = batches.size();
+    for (int i = 0; i < 6; ++i)
+      avg[i] /= n;
+    if (rmsd) {
+      for (int i = 0; i < 6; ++i)
+        rmsd[i] = 0.;
+      for (const Batch& batch : batches)
+        for (int i = 0; i < 6; ++i)
+          rmsd[i] += sq(avg[i] - batch.floats[i]);
+      for (int i = 0; i < 6; ++i)
+        rmsd[i] = std::sqrt(rmsd[i] / n);
+    }
+    return UnitCell(avg[0], avg[1], avg[2], avg[3], avg[4], avg[5]);
+  }
+
   Dataset& last_dataset() {
     if (datasets.empty())
       fail("MTZ dataset not found (missing DATASET header line?).");
@@ -266,6 +292,13 @@ struct Mtz {
         ++n;
     return n;
   }
+  int count_type(char type) const {
+    int n = 0;
+    for (const Column& col : columns)
+      if (col.type == type)
+        ++n;
+    return n;
+  }
   Column* column_with_label(const std::string& label,
                             const Dataset* ds=nullptr) {
     for (Column& col : columns)
@@ -291,6 +324,14 @@ struct Mtz {
     return cols;
   }
 
+  const Column* column_with_one_of_labels(std::initializer_list<const char*> labels) const {
+    for (const char* label : labels) {
+      if (const Column* col = column_with_label(label))
+        return col;
+    }
+    return nullptr;
+  }
+
   bool has_data() const {
     return data.size() == columns.size() * nreflections;
   }
@@ -372,12 +413,12 @@ struct Mtz {
   }
 
   static UnitCell read_cell_parameters(const char* line) {
-    double a = simple_atof(line, &line);
-    double b = simple_atof(line, &line);
-    double c = simple_atof(line, &line);
-    double alpha = simple_atof(line, &line);
-    double beta = simple_atof(line, &line);
-    double gamma = simple_atof(line, &line);
+    double a = fast_atof(line, &line);
+    double b = fast_atof(line, &line);
+    double c = fast_atof(line, &line);
+    double alpha = fast_atof(line, &line);
+    double beta = fast_atof(line, &line);
+    double gamma = fast_atof(line, &line);
     return UnitCell(a, b, c, alpha, beta, gamma);
   }
 
@@ -388,9 +429,9 @@ struct Mtz {
 
   template<typename Stream>
   void seek_headers(Stream& stream) {
-    if (!stream.seek(4 * (header_offset - 1)))
-      fail("Cannot rewind to the MTZ header at byte "
-           + std::to_string(header_offset));
+    std::ptrdiff_t pos = 4 * std::ptrdiff_t(header_offset - 1);
+    if (!stream.seek(pos))
+      fail("Cannot rewind to the MTZ header at byte " + std::to_string(pos));
   }
 
   // read headers until END
@@ -443,13 +484,13 @@ struct Mtz {
           symops.push_back(parse_triplet(args));
           break;
         case ialpha4_id("RESO"):
-          min_1_d2 = simple_atof(args, &args);
-          max_1_d2 = simple_atof(args, &args);
+          min_1_d2 = fast_atof(args, &args);
+          max_1_d2 = fast_atof(args, &args);
           break;
         case ialpha4_id("VALM"):
           if (*args != 'N') {
             const char* endptr;
-            float v = (float) simple_atof(args, &endptr);
+            float v = (float) fast_atof(args, &endptr);
             if (*endptr == '\0' || is_space(*endptr))
               valm = v;
             else
@@ -461,8 +502,8 @@ struct Mtz {
           Column& col = columns.back();
           col.label = read_word(args, &args);
           col.type = read_word(args, &args)[0];
-          col.min_value = (float) simple_atof(args, &args);
-          col.max_value = (float) simple_atof(args, &args);
+          col.min_value = (float) fast_atof(args, &args);
+          col.max_value = (float) fast_atof(args, &args);
           col.dataset_id = simple_atoi(args);
           col.parent = this;
           col.idx = columns.size() - 1;
@@ -507,7 +548,7 @@ struct Mtz {
         // case("DRES"): not in use yet
         case ialpha4_id("DWAV"):
           if (simple_atoi(args, &args) == last_dataset().id)
-            datasets.back().wavelength = simple_atof(args);
+            datasets.back().wavelength = fast_atof(args);
           else
             warn("MTZ DWAV line: unusual numbering.");
           break;
@@ -570,6 +611,7 @@ struct Mtz {
         }
       }
     }
+    appended_text = stream.read_rest();
   }
 
   void setup_spacegroup() {
@@ -619,6 +661,7 @@ struct Mtz {
   void read_file(const std::string& path) {
     fileptr_t f = file_open(path.c_str(), "rb");
     try {
+      source_path = path;
       read_stream(FileStream{f.get()}, true);
     } catch (std::runtime_error& e) {
       fail(std::string(e.what()) + ": " + path);
@@ -627,11 +670,11 @@ struct Mtz {
 
   template<typename Input>
   void read_input(Input&& input, bool with_data) {
+    source_path = input.path();
     if (input.is_stdin()) {
       read_stream(FileStream{stdin}, with_data);
-    } else if (std::unique_ptr<char[]> mem = input.memory()) {
-      MemoryStream stream(mem.get(), mem.get() + input.memory_size());
-      read_stream(std::move(stream), with_data);
+    } else if (CharArray mem = input.uncompress_into_buffer()) {
+      read_stream(MemoryStream(mem.data(), mem.size()), with_data);
     } else {
       fileptr_t f = file_open(input.path().c_str(), "rb");
       read_stream(FileStream{f.get()}, true);
@@ -759,7 +802,9 @@ struct Mtz {
   }
 
   // Function for writing MTZ file
-  void write_to_stream(std::FILE* stream) const;
+  void write_to_cstream(std::FILE* stream) const;
+  void write_to_string(std::string& str) const;
+  template<typename Stream> void write_to_stream(Stream stream) const;
   void write_to_file(const std::string& path) const;
 };
 
@@ -824,11 +869,12 @@ namespace gemmi {
     int len = gf_snprintf(buf, 81, __VA_ARGS__); \
     if (len < 80) \
       std::memset(buf + len, ' ', 80 - len); \
-    if (std::fwrite(buf, 80, 1, stream) != 1) \
+    if (write(buf, 80, 1) != 1) \
       fail("Writing MTZ file failed"); \
   } while(0)
 
-void Mtz::write_to_stream(std::FILE* stream) const {
+template<typename Write>
+void Mtz::write_to_stream(Write write) const {
   // uses: data, spacegroup, nreflections, batches, cell, sort_order,
   //       valm, columns, datasets, history
   if (!has_data())
@@ -840,8 +886,8 @@ void Mtz::write_to_stream(std::FILE* stream) const {
   std::memcpy(buf + 4, &header_start, 4);
   std::int32_t machst = is_little_endian() ? 0x00004144 : 0x11110000;
   std::memcpy(buf + 8, &machst, 4);
-  if (std::fwrite(buf, 80, 1, stream) != 1 ||
-      std::fwrite(data.data(), 4, data.size(), stream) != data.size())
+  if (write(buf, 80, 1) != 1 ||
+      write(data.data(), 4, data.size()) != data.size())
     fail("Writing MTZ file failed");
   WRITE("VERS MTZ:V1.1");
   WRITE("TITLE %s", title.c_str());
@@ -904,7 +950,7 @@ void Mtz::write_to_stream(std::FILE* stream) const {
       for (size_t j = i; j < std::min(batches.size(), i + 12); ++j, pos += 6)
         gf_snprintf(buf + pos, 7, "%6zu", j + 1);
       std::memset(buf + pos, ' ', 80 - pos);
-      if (std::fwrite(buf, 80, 1, stream) != 1)
+      if (write(buf, 80, 1) != 1)
         fail("Writing MTZ file failed");
     }
   }
@@ -918,17 +964,16 @@ void Mtz::write_to_stream(std::FILE* stream) const {
   }
   if (!batches.empty()) {
     WRITE("MTZBATS");
-    int n = 0;
     for (const Batch& batch : batches) {
       // keep the numbers the same as in files written by libccp4
       WRITE("BH %8d %7zu %7zu %7zu",
-            ++n, batch.ints.size() + batch.floats.size(),
+            batch.number, batch.ints.size() + batch.floats.size(),
             batch.ints.size(), batch.floats.size());
       WRITE("TITLE %.70s", batch.title.c_str());
       if (batch.ints.size() != 29 || batch.floats.size() != 156)
         fail("wrong size of binaries batch headers");
-      std::fwrite(batch.ints.data(), 4, batch.ints.size(), stream);
-      std::fwrite(batch.floats.data(), 4, batch.floats.size(), stream);
+      write(batch.ints.data(), 4, batch.ints.size());
+      write(batch.floats.data(), 4, batch.floats.size());
       WRITE("BHCH  %7.7s %7.7s %7.7s",
             batch.axes.size() > 0 ? batch.axes[0].c_str() : "",
             batch.axes.size() > 1 ? batch.axes[1].c_str() : "",
@@ -936,14 +981,31 @@ void Mtz::write_to_stream(std::FILE* stream) const {
     }
   }
   WRITE("MTZENDOFHEADERS");
+  if (!appended_text.empty()) {
+    if (write(appended_text.data(), appended_text.size(), 1) != 1)
+      fail("Writing MTZ file failed");
+  }
 }
 
 #undef WRITE
 
+void Mtz::write_to_cstream(std::FILE* stream) const {
+  write_to_stream([&](const void *ptr, size_t size, size_t nmemb) {
+      return std::fwrite(ptr, size, nmemb, stream);
+  });
+}
+
+void Mtz::write_to_string(std::string& str) const {
+  write_to_stream([&](const void *ptr, size_t size, size_t nmemb) {
+      str.append(static_cast<const char*>(ptr), size * nmemb);
+      return nmemb;
+  });
+}
+
 void Mtz::write_to_file(const std::string& path) const {
   fileptr_t f = file_open(path.c_str(), "wb");
   try {
-    return write_to_stream(f.get());
+    return write_to_cstream(f.get());
   } catch (std::runtime_error& e) {
     fail(std::string(e.what()) + ": " + path);
   }
