@@ -1,6 +1,6 @@
 // Copyright 2017 Global Phasing Ltd.
 //
-// Structure -> cif::Document -> mmcif (PDBx/mmCIF) file
+// Create cif::Document (for PDBx/mmCIF file) from Structure.
 
 #ifndef GEMMI_TO_MMCIF_HPP_
 #define GEMMI_TO_MMCIF_HPP_
@@ -44,7 +44,8 @@ struct MmcifOutputGroups {
 
   explicit MmcifOutputGroups(bool all)
     : atoms(all), block_name(all), entry(all), database_status(all),
-      cell(all), symmetry(all), entity(all), entity_poly(all),
+      cell(all), symmetry(all), entity(all),
+      entity_poly(false),  // see the comment under "if (groups.entity_poly)"
       struct_ref(all), chem_comp(all), exptl(all), diffrn(all),
       reflns(all), refine(all), title_keywords(all), ncs(all),
       struct_asym(all), origx(all), struct_conf(all), struct_sheet(all),
@@ -370,6 +371,10 @@ void write_struct_conn(const Structure& st, cif::Block& block) {
   // example:
   // disulf1 disulf A CYS 3  SG ? 3 ? 1_555 A CYS 18 SG ? 18 ?  1_555 ? 2.045
   std::array<bool,(int)Connection::Type::Unknown+1> type_ids{};
+  bool use_ccp4_link_id = false;
+  for (const Connection& con : st.connections)
+    if (!con.link_id.empty())
+      use_ccp4_link_id = true;
   cif::Loop& conn_loop = block.init_mmcif_loop("_struct_conn.",
       {"id", "conn_type_id",
        "ptnr1_auth_asym_id", "ptnr1_label_asym_id", "ptnr1_label_comp_id",
@@ -378,47 +383,47 @@ void write_struct_conn(const Structure& st, cif::Block& block) {
        "ptnr2_auth_asym_id", "ptnr2_label_asym_id", "ptnr2_label_comp_id",
        "ptnr2_label_seq_id", "ptnr2_label_atom_id", "pdbx_ptnr2_label_alt_id",
        "ptnr2_auth_seq_id", "pdbx_ptnr2_PDB_ins_code", "ptnr2_symmetry",
-       "details", "pdbx_dist_value", "ccp4_link_id"});
+       "details", "pdbx_dist_value"});
+  if (use_ccp4_link_id)
+    conn_loop.tags.push_back("_struct_conn.ccp4_link_id");
   for (const Connection& con : st.connections) {
-    const Residue* res1 = st.models[0].find_residue(con.partner1.chain_name, con.partner1.res_id);
-    const Residue* res2 = st.models[0].find_residue(con.partner2.chain_name, con.partner2.res_id);
-    if (!res1 || !res2)
+    const_CRA cra1 = st.models[0].find_cra(con.partner1);
+    const_CRA cra2 = st.models[0].find_cra(con.partner2);
+    if (!cra1.residue || !cra2.residue)
       continue;
-    const Atom* at1 = res1->find_atom(con.partner1.atom_name, con.partner1.altloc);
-    const Atom* at2 = res2->find_atom(con.partner2.atom_name, con.partner2.altloc);
-
+    const Atom* at1 = cra1.atom;
+    const Atom* at2 = cra2.atom;
     std::string im_pdb_symbol = "?", im_dist_str = "?";
     if (at1 && at2) {
-      SymImage im = st.cell.find_nearest_image(at1->pos,
-                                               at2->pos, con.asu);
+      SymImage im = st.cell.find_nearest_image(at1->pos, at2->pos, con.asu);
       im_pdb_symbol = im.pdb_symbol(true);
       im_dist_str = to_str_prec<4>(im.dist());
     }
-    conn_loop.add_row({
-        con.name,                                        // id
-        connection_type_to_string(con.type),             // conn_type_id
-        impl::qchain(con.partner1.chain_name),           // ptnr1_auth_asym_id
-        subchain_or_dot(*res1),                          // ptnr1_label_asym_id
-        res1->name,                                      // ptnr1_label_comp_id
-        res1->label_seq.str('.'),                        // ptnr1_label_seq_id
-        at1 ? cif::quote(at1->name) : "?",               // ptnr1_label_atom_id
-        at1 ? std::string(1, at1->altloc_or('?')) : "?", // pdbx_ptnr1_label_alt_id
-        res1->seqid.num.str(),                           // ptnr1_auth_seq_id
-        pdbx_icode(con.partner1.res_id),                 // ptnr1_PDB_ins_code
-        "1_555",                                         // ptnr1_symmetry
-        impl::qchain(con.partner2.chain_name),           // ptnr2_auth_asym_id
-        subchain_or_dot(*res2),                          // ptnr2_label_asym_id
-        res2->name,                                      // ptnr2_label_comp_id
-        res2->label_seq.str('.'),                        // ptnr2_label_seq_id
-        at2 ? cif::quote(at2->name) : "?",               // ptnr2_label_atom_id
-        at2 ? std::string(1, at2->altloc_or('?')) : "?", // pdbx_ptnr2_label_alt_id
-        res2->seqid.num.str(),                           // ptnr2_auth_seq_id
-        pdbx_icode(con.partner2.res_id),                 // ptnr2_PDB_ins_code
-        im_pdb_symbol,                                   // ptnr2_symmetry
-        "?",                                             // details
-        im_dist_str,                                     // pdbx_dist_value
-        impl::string_or_qmark(con.link_id)               // ccp4_link_id
-    });
+    auto& v = conn_loop.values;
+    v.emplace_back(impl::string_or_qmark(con.name));        // id
+    v.emplace_back(connection_type_to_string(con.type));    // conn_type_id
+    v.emplace_back(impl::qchain(con.partner1.chain_name));  // ptnr1_auth_asym_id
+    v.emplace_back(subchain_or_dot(*cra1.residue));         // ptnr1_label_asym_id
+    v.emplace_back(cra1.residue->name);                     // ptnr1_label_comp_id
+    v.emplace_back(cra1.residue->label_seq.str('.'));       // ptnr1_label_seq_id
+    v.emplace_back(at1 ? cif::quote(at1->name) : "?");      // ptnr1_label_atom_id
+    v.emplace_back(1, at1 ? at1->altloc_or('?') : '?');     // pdbx_ptnr1_label_alt_id
+    v.emplace_back(cra1.residue->seqid.num.str());          // ptnr1_auth_seq_id
+    v.emplace_back(pdbx_icode(con.partner1.res_id));        // ptnr1_PDB_ins_code
+    v.emplace_back("1_555");                                // ptnr1_symmetry
+    v.emplace_back(impl::qchain(con.partner2.chain_name));  // ptnr2_auth_asym_id
+    v.emplace_back(subchain_or_dot(*cra2.residue));         // ptnr2_label_asym_id
+    v.emplace_back(cra2.residue->name);                     // ptnr2_label_comp_id
+    v.emplace_back(cra2.residue->label_seq.str('.'));       // ptnr2_label_seq_id
+    v.emplace_back(at2 ? cif::quote(at2->name) : "?");      // ptnr2_label_atom_id
+    v.emplace_back(1, at2 ? at2->altloc_or('?') : '?');     // pdbx_ptnr2_label_alt_id
+    v.emplace_back(cra2.residue->seqid.num.str());          // ptnr2_auth_seq_id
+    v.emplace_back(pdbx_icode(con.partner2.res_id));        // ptnr2_PDB_ins_code
+    v.emplace_back(im_pdb_symbol);                          // ptnr2_symmetry
+    v.emplace_back("?");                                    // details
+    v.emplace_back(im_dist_str);                            // pdbx_dist_value
+    if (use_ccp4_link_id)
+      v.emplace_back(impl::string_or_qmark(con.link_id));   // ccp4_link_id
     type_ids[int(con.type)] = true;
   }
 
@@ -462,6 +467,12 @@ void write_ncs_oper(const Structure& st, cif::Block& block) {
   for (const NcsOp& op : st.ncs)
     add_op(op);
 }
+
+bool is_valid_block_name(const std::string& name) {
+  return !name.empty() &&
+         std::all_of(name.begin(), name.end(), [](char c){ return c >= '!' && c <= '~'; });
+}
+
 } // namespace impl
 
 void update_mmcif_block(const Structure& st, cif::Block& block, MmcifOutputGroups groups) {
@@ -470,7 +481,7 @@ void update_mmcif_block(const Structure& st, cif::Block& block, MmcifOutputGroup
     return;
 
   if (groups.block_name)
-    block.name = (!st.name.empty() ? st.name : "xxxx");
+    block.name = impl::is_valid_block_name(st.name) ? st.name : "model";
 
   auto e_id = st.info.find("_entry.id");
   std::string id = cif::quote(e_id != st.info.end() ? e_id->second : block.name);
@@ -515,6 +526,12 @@ void update_mmcif_block(const Structure& st, cif::Block& block, MmcifOutputGroup
   }
 
   if (groups.entity_poly) {
+    // In this category we write mandatory tags only.
+    // But if such a file is deposited to the wwPDB it causes:
+    //  ERROR: Missing '_entity_poly.pdbx_strand_id' item.
+    // If pdbx_strand_id was present, OneDep would complain about missing
+    // _entity_poly.pdbx_seq_one_letter_code. So just in case, by default
+    // MmcifOutputGroups::entity_poly is false.
     cif::Loop& ent_poly_loop = block.init_mmcif_loop("_entity_poly.", {"entity_id", "type"});
     for (const Entity& ent : st.entities)
       if (ent.entity_type == EntityType::Polymer)
@@ -577,6 +594,9 @@ void update_mmcif_block(const Structure& st, cif::Block& block, MmcifOutputGroup
       for (const Chain& chain : model.chains)
         for (const Residue& res : chain.residues)
           resnames.insert(res.name);
+    for (const Entity& ent : st.entities)
+      for (const std::string& item : ent.full_sequence)
+        resnames.insert(Entity::first_mon(item));
     cif::Loop& chem_comp_loop = block.init_mmcif_loop("_chem_comp.", {"id", "type"});
     for (const std::string& name : resnames)
       chem_comp_loop.add_row({name, "."});
@@ -929,7 +949,8 @@ void update_mmcif_block(const Structure& st, cif::Block& block, MmcifOutputGroup
     cif::Loop& sheet_loop = block.init_mmcif_loop("_struct_sheet.",
                                                   {"id", "number_strands"});
     for (const Sheet& sheet : st.sheets)
-      sheet_loop.add_row({sheet.name, std::to_string(sheet.strands.size())});
+      sheet_loop.add_row({impl::string_or_dot(sheet.name),
+                          std::to_string(sheet.strands.size())});
 
     cif::Loop& order_loop = block.init_mmcif_loop("_struct_sheet_order.",
                     {"sheet_id", "range_id_1", "range_id_2", "sense"});
@@ -937,7 +958,7 @@ void update_mmcif_block(const Structure& st, cif::Block& block, MmcifOutputGroup
       for (size_t i = 1; i < sheet.strands.size(); ++i) {
         const Sheet::Strand& strand = sheet.strands[i];
         if (strand.sense != 0)
-          order_loop.add_row({sheet.name,
+          order_loop.add_row({impl::string_or_dot(sheet.name),
                               std::to_string(i), std::to_string(i+1),
                               strand.sense > 0 ? "parallel" : "anti-parallel"});
       }
@@ -956,7 +977,7 @@ void update_mmcif_block(const Structure& st, cif::Block& block, MmcifOutputGroup
         if (!cra1.residue || !cra2.residue)
           continue;
         range_loop.add_row({
-          sheet.name,                                 // sheet_id
+          impl::string_or_dot(sheet.name),            // sheet_id
           std::to_string(i+1),                        // id
           impl::qchain(cra1.chain->name),             // beg_auth_asym_id
           impl::subchain_or_dot(*cra1.residue),       // beg_label_asym_id
@@ -992,7 +1013,7 @@ void update_mmcif_block(const Structure& st, cif::Block& block, MmcifOutputGroup
         if (!cra1.residue || !cra2.residue)
           continue;
         hbond_loop.add_row({
-          sheet.name,                                 // sheet_id
+          impl::string_or_dot(sheet.name),            // sheet_id
           std::to_string(i),                          // range_id_1
           std::to_string(i+1),                        // range_id_2
           impl::qchain(cra1.chain->name),             // range_1_auth_asym_id
