@@ -2,11 +2,13 @@
 
 #include "common.h"
 #include <pybind11/stl.h>
+#include <pybind11/numpy.h>
 #include "gemmi/version.hpp"
 #include "gemmi/dirwalk.hpp"
 #include "gemmi/fileutil.hpp"  // for expand_if_pdb_code
 #include "gemmi/small.hpp"    // for SmallStructure
 #include "gemmi/interop.hpp"  // for atom_to_site, mx_to_sx_structure
+#include "gemmi/bessel.hpp"   // for bessel_i1_over_i0
 
 namespace py = pybind11;
 
@@ -71,6 +73,7 @@ void add_misc(py::module& m) {
   m.def("expand_if_pdb_code", &gemmi::expand_if_pdb_code,
         py::arg("code"), py::arg("filetype")='M');
   m.attr("hc") = py::float_(gemmi::hc());
+  m.def("bessel_i1_over_i0", py::vectorize(gemmi::bessel_i1_over_i0));
 }
 
 PYBIND11_MODULE(gemmi, mg) {
@@ -93,6 +96,7 @@ PYBIND11_MODULE(gemmi, mg) {
   add_hkl(mg);
   add_mtz(mg);
   add_monlib(mg);
+  add_topo(mg);
   add_alignment(mg);
   add_select(mg);
   add_search(mg);
