@@ -86,6 +86,7 @@ void add_meta(py::module& m) {
                      " |shift|=", self.tr.vec.length(),
                      (self.given ? " (" : " (not "), "given)>");
     });
+  py::bind_vector<std::vector<NcsOp>>(m, "NcsOpList");
 
   py::enum_<EntityType>(m, "EntityType")
     .value("Unknown", EntityType::Unknown)
@@ -115,7 +116,7 @@ void add_meta(py::module& m) {
     .def_readwrite("full_sequence", &Entity::full_sequence)
     .def_static("first_mon", &Entity::first_mon)
     .def("__repr__", [](const Entity& self) { return tostr(self); });
-
+  py::bind_vector<std::vector<Entity>>(m, "EntityList");
 
   py::enum_<Connection::Type>(m, "ConnectionType")
     .value("Covale", Connection::Type::Covale)
@@ -137,27 +138,38 @@ void add_meta(py::module& m) {
         return tostr("<gemmi.Connection ", self.name, "  ",
                      self.partner1.str(), " - ", self.partner2.str(), '>');
     });
+  py::bind_vector<std::vector<Connection>>(m, "ConnectionList");
 
 
   py::class_<Assembly> assembly(m, "Assembly");
   py::class_<Assembly::Operator>(assembly, "Operator")
     .def(py::init<>())
-    .def_readonly("name", &Assembly::Operator::name)
-    .def_readonly("type", &Assembly::Operator::type)
-    .def_readonly("transform", &Assembly::Operator::transform);
+    .def_readwrite("name", &Assembly::Operator::name)
+    .def_readwrite("type", &Assembly::Operator::type)
+    .def_readwrite("transform", &Assembly::Operator::transform);
   py::bind_vector<std::vector<Assembly::Operator>>(assembly, "OperatorList");
 
   py::class_<Assembly::Gen>(assembly, "Gen")
     .def(py::init<>())
-    .def_readonly("chains", &Assembly::Gen::chains)
-    .def_readonly("subchains", &Assembly::Gen::subchains)
+    .def_readwrite("chains", &Assembly::Gen::chains)
+    .def_readwrite("subchains", &Assembly::Gen::subchains)
     .def_readonly("operators", &Assembly::Gen::operators);
   py::bind_vector<std::vector<Assembly::Gen>>(assembly, "GenList");
 
   assembly
-    .def_readonly("name", &Assembly::name)
-    .def_readonly("oligomeric_details", &Assembly::oligomeric_details)
+    .def(py::init<const std::string&>())
+    .def_readwrite("name", &Assembly::name)
+    .def_readwrite("oligomeric_details", &Assembly::oligomeric_details)
     .def_readonly("generators", &Assembly::generators)
+    .def_readwrite("special_kind", &Assembly::special_kind)
     ;
   py::bind_vector<std::vector<Assembly>>(m, "AssemblyList");
+
+  py::enum_<Assembly::SpecialKind>(m, "AssemblySpecialKind")
+    .value("NA", Assembly::SpecialKind::NA)
+    .value("CompleteIcosahedral", Assembly::SpecialKind::CompleteIcosahedral)
+    .value("RepresentativeHelical", Assembly::SpecialKind::RepresentativeHelical)
+    .value("CompletePoint", Assembly::SpecialKind::CompletePoint);
+
+
 }
