@@ -14,13 +14,9 @@
 namespace py = pybind11;
 using namespace gemmi;
 
-PYBIND11_MAKE_OPAQUE(std::vector<Connection>)
-PYBIND11_MAKE_OPAQUE(std::vector<NcsOp>)
-PYBIND11_MAKE_OPAQUE(std::vector<Entity>)
 using info_map_type = std::map<std::string, std::string>;
 PYBIND11_MAKE_OPAQUE(info_map_type)
 
-
 template<typename T, typename C>
 C& add_item(T& container, C child, int pos) {
   if ((size_t) pos > container.size()) // true also for negative pos
@@ -79,12 +75,8 @@ void add_mol(py::module& m) {
     .value("Mmjson", CoorFormat::Mmjson)
     .value("ChemComp", CoorFormat::ChemComp);
 
-  py::bind_vector<std::vector<Connection>>(m, "ConnectionList");
-  py::bind_vector<std::vector<NcsOp>>(m, "NcsOpList");
-  py::bind_vector<std::vector<Entity>>(m, "EntityList");
   py::bind_map<info_map_type>(m, "InfoMap");
 
-
   py::class_<Structure> structure(m, "Structure");
   structure
     .def(py::init<>())
@@ -366,7 +358,7 @@ void add_mol(py::module& m) {
     .def("__len__", &AtomGroup::size)
     .def("__iter__", [](AtomGroup& g) { return py::make_iterator(g); },
          py::keep_alive<0, 1>())
-    .def("__bool__", [](const ResidueSpan &g) -> bool { return !g.empty(); })
+    .def("__bool__", [](const AtomGroup &g) -> bool { return !g.empty(); })
     .def("__getitem__", [](AtomGroup& g, int index) -> Atom& {
         return g[normalize_index(index, g)];
     }, py::arg("index"), py::return_value_policy::reference_internal)
@@ -468,6 +460,10 @@ void add_mol(py::module& m) {
         "Input: four points. Output: dihedral angle in radians.");
   m.def("calculate_phi_psi", &calculate_phi_psi,
         py::arg("prev_residue"), py::arg("residue"), py::arg("next_residue"));
+  m.def("find_best_plane", &find_best_plane,
+        py::arg("atoms"));
+  m.def("get_distance_from_plane", &get_distance_from_plane,
+        py::arg("pos"), py::arg("coeff"));
   m.def("calculate_omega", &calculate_omega,
         py::arg("residue"), py::arg("next_residue"));
   m.def("calculate_sequence_weight", &calculate_sequence_weight,
