@@ -44,12 +44,18 @@ template<> void add_to_asu_data(py::class_<AsuData<std::complex<float>>>& cl) {
          py::arg("exact_size")=std::array<int,3>{{0,0,0}},
          py::arg("order")=AxisOrder::XYZ);
 }
+template<> void add_to_asu_data(py::class_<AsuData<float>>& cl) {
+  using AsuData = AsuData<float>;
+  cl.def("calculate_correlation", [](const AsuData& self, const AsuData& other) {
+      return calculate_hkl_value_correlation(self.v, other.v);
+  });
+}
 
 template<typename T>
 void add_asudata(py::module& m, const std::string& name) {
   py::class_<HklValue<T>>(m, (name + "HklValue").c_str())
     .def_readonly("hkl", &HklValue<T>::hkl)
-    .def_readonly("value", &HklValue<T>::value)
+    .def_readwrite("value", &HklValue<T>::value)
     .def("__repr__", [name](const HklValue<T>& self) {
         return tostr("<gemmi.", name, "HklValue (",
                      self.hkl[0], ',', self.hkl[1], ',', self.hkl[2], ") ",
@@ -169,6 +175,11 @@ void add_recgrid(py::module& m, const std::string& name) {
 
 void add_recgrid(py::module& m) {
   using VS = ValueSigma<float>;
+  try {
+    PYBIND11_NUMPY_DTYPE(VS, value, sigma);
+  } catch (py::error_already_set &) {
+    // numpy can't be imported, that's ok
+  }
   py::class_<VS>(m, "ValueSigma")
     .def_readwrite("value", &VS::value)
     .def_readwrite("sigma", &VS::sigma)
