@@ -11,10 +11,16 @@
 namespace py = pybind11;
 using namespace gemmi;
 
-PYBIND11_MAKE_OPAQUE(std::vector<Op::Tran>)
-
 void add_symmetry(py::module& m) {
-  py::bind_vector<std::vector<Op::Tran>>(m, "TranList");
+  py::enum_<CrystalSystem>(m, "CrystalSystem")
+    .value("Triclinic", CrystalSystem::Triclinic)
+    .value("Monoclinic", CrystalSystem::Monoclinic)
+    .value("Orthorhombic", CrystalSystem::Orthorhombic)
+    .value("Tetragonal", CrystalSystem::Tetragonal)
+    .value("Trigonal", CrystalSystem::Trigonal)
+    .value("Hexagonal", CrystalSystem::Hexagonal)
+    .value("Cubic", CrystalSystem::Cubic)
+    ;
   py::class_<Op>(m, "Op")
     .def(py::init<>(&Op::identity))
     .def(py::init(&parse_triplet))
@@ -161,6 +167,7 @@ void add_symmetry(py::module& m) {
     .def("laue_str", &SpaceGroup::laue_str,
          "Returns name of the Laue class (for centrosymmetric groups "
          "the same as point_group_hm).")
+    .def("crystal_system", &SpaceGroup::crystal_system)
     .def("crystal_system_str", &SpaceGroup::crystal_system_str,
          "Returns lower-case name of the crystal system.")
     .def("is_reference_setting", &SpaceGroup::is_reference_setting)
@@ -179,7 +186,15 @@ void add_symmetry(py::module& m) {
     }, py::arg("miller_array").noconvert())
     .def("__repr__", [](const SpaceGroup &self) {
         return "<gemmi.SpaceGroup(\"" + self.xhm() + "\")>";
-    });
+    })
+    .def(py::pickle(
+        [](const SpaceGroup &self) {
+          return self.xhm();
+        },
+        [](const std::string &s) {
+          return const_cast<SpaceGroup*>(&get_spacegroup_by_name(s));
+        }
+    ));
 
   py::class_<ReciprocalAsu>(m, "ReciprocalAsu")
     .def(py::init<const SpaceGroup*>())
